//
// PROGRAM HODINY REÁLNÉHO ČASU, SYNCHRONIZACE PŘES NTP SERVER
//
//   David Kohout a Tomáš Kohout 10/2022
//    Kompaktibilní s Wemos D1 R1 ( R2 )
// RTC Hodiny reálného času DS3231 AT24C32 IIC
//       I2C Display LCD 1602 16x2

#include <LiquidCrystal_I2C.h>     // Knihovna LCD displeje
#include <RTClib.h>                // Knihovna reálných hodin DS3231
#include <ESP8266WiFi.h>           // WiFi ESP 8266
#include <NTPClient.h>             // NTP klient ( reálný čas z pool.ntp.org )
#include <WiFiUdp.h>               // Knihovna pro Arduino WiFi
#include <Ticker.h>                // Nahradí delay() neblokujícími funkcemi

LiquidCrystal_I2C lcd(0x27, 16, 2); // Nastav adresu LCD na 0x27 pro 16 znaků a 2 řádky displeje
RTC_DS1307 DS1307;                  // Nakonfigurování hodin RTC
Ticker takt ;                       // "Časovač" pro takt
Ticker zapis;                       // "Časovač" pro zápis
WiFiUDP ntpUDP;                     // Odesílat a přijímat zprávy UDP
NTPClient timeClient(ntpUDP);       // NTP klient

char seznamDni[7][8] = {"NE", "PO", "UT", "ST", "CT", "PA", "SO"}; // vytvoření pole seznamu dní s názvy jednotlivých dní
const char *ssid     = "JMENOWIFI";         // Jméno WiFi sítě
const char *password = "HESLOWIFI";         // Heslo k WiFi
String formattedDate;                       // Řetězec znaků - datum a čas
String dayStamp;                            // Časové razítko - den
String timeStamp;                           // Časové razítko - čas
unsigned long previousMillis = 0;           // Počítadlo ms 
unsigned long intervalwifi = 30000;         // Interval pokus připojení k WiFi
int pocet;                                  // Pomocný čítač - počet připojení k WiFi
bool synchronizace;                         // Bit synchronizace proběhla
int taktu;                                  // Pomocný čítač - cykly CPU



void initWiFi() {                           // Inicializace WiFi
  WiFi.mode(WIFI_STA);
}

void setup() {
  
  pinMode (D10, OUTPUT);            // LED Wi-Fi připojena
  pinMode (D9, OUTPUT);             // Synchronizace času provedena
  lcd.init();                       // Inicializace LCD displeje
  lcd.backlight();                  // Rozsvit podsviceni LCD displeje
  Serial.begin(9600);               // Otevreni serioveho portu,rychlost 9.6 kB
  // Připojení k WiFi
  initWiFi();                       // Zavolání funkce inicializace WiFi

  
//  Každých 60 vteřin spusť funkci k vyčtení dat z NTP serveru
    takt.attach ( 60.0, zapisdat ) ;


// Inicializace NTP klienta
   timeClient.begin();
  
  // Nastavení offsetu ve vteřinách pro vaši časovou zónu, za // příklady
  // Česká republika - GMT +2 (7200 vteřin )
 
  //timeClient.setTimeOffset(7200); // 7200 = Česká republika GMT +2  // letní čas
    timeClient.setTimeOffset(3600); // 3600 = Česká republika GMT +1  // zimní čas
}


void loop() {

    unsigned long currentMillis = millis();
  
  // Pokud WiFi není připojena, pokus se znovu připojit každých - intervalwifi
  
  if ((WiFi.status() != WL_CONNECTED) && (currentMillis - previousMillis >=intervalwifi)) {
    Serial.print(millis());
    Serial.println("Znovu pripojeni k WiFi...");
 
    WiFi.reconnect();
    previousMillis = currentMillis;
  }
  
if (WiFi.status() == WL_CONNECTED) {
   while(!timeClient.update()) {
    timeClient.forceUpdate();
    pocet=pocet+1;
  }
  }
  
  

  DateTime datumCas = DS1307.now();         // načtení aktuálního času a data do proměnné datumCas

  if (datumCas.hour() < 10) {               // Pokud je hodina menší jak 10,otevři větev
    lcd.setCursor(4, 1);                    // Nastav LCD na 1 řádek,4 pozice
    lcd.print("0");                         // Zapiš 0 před hodiny na LCD
    lcd.print(datumCas.hour());             // Zapiš hodiny na LCD
    lcd.print(":");                         // Zapiš znak : na LCD
  }                                         // Ukonči větev
  if (datumCas.hour() > 9) {                // Nebo je-li hodina větší jak 9,otevři větev

    lcd.setCursor(4, 1);                    // Nastav LCD na 1 řádek,4 pozice
    lcd.print(datumCas.hour());             // Zapis na LCD hodiny
    lcd.print(":");                         // Zapis na LCD znak :
  }                                         // Ukonci vetev

  if (datumCas.minute() < 10) {             // Pokud je minuta menší jak 10 otevři větev
    lcd.setCursor(7, 1);                    // Nastav LCD na 1 řádek,7 pozice
    lcd.print("0");                         // Zapiš 0 před minuty na LCD
    lcd.print(datumCas.minute());           // Zapiš minuty na LCD
    lcd.print(":");                         // Zapiš znak : na LCD
  }                                         // Ukonči větev
  if (datumCas.minute() > 9) {              // Nebo je-li minuta větší jak 9 otevři větev

    lcd.setCursor(7, 1);                    // Nastav LCD na 1 radek,7 pozice
    lcd.print(datumCas.minute());           // Zapiš minuty na LCD
    lcd.print(":");                         // Zapiš znak : na LCD
  }                                         // Ukonči větev

  if (datumCas.second() < 10) {             // Pokud je vteřina menší jak 10 otevři větev

    lcd.setCursor(10, 1);                   // Nastav LCD na 1 radek,10 pozice
    lcd.print("0");                         // Zapiš 0 před vteřiny na LCD
    lcd.print(datumCas.second());           // Zapiš vteřiny na LCD
  }                                         // Ukonči větev
  if (datumCas.second() > 9) {              // Nebo je-li vteřina větší jak 9 otevři větev

    lcd.setCursor(10, 1);                   // Nastav LCD na 1 řádek,10 pozice
    lcd.print(datumCas.second());           // Zapiš vteřiny na LCD
  }
  lcd.setCursor(1, 0);                             // Nastav LCD na 0 řádek,1 pozice
  lcd.print(seznamDni[datumCas.dayOfTheWeek()]);   // Zapiš den v týdnu

  if (datumCas.day() < 10) {                      // Pokud je den v měsíci menší jak 10, otevři větev
    lcd.setCursor(5, 0);                          // Nastav LCD na 0 řádek, 5 pozice
    lcd.print("0");                               // Zapiš 0 pred den na LCD
    lcd.print(datumCas.day());                    // Zapiš den v měsíci na LCD
  }                                               // Ukonči větev
  if (datumCas.day() > 9) {                       // Nebo je-li den větší jak 9 otevři větev
    lcd.setCursor(5, 0);                          // Nastav LCD na 0 řádek, 5 pozice
    lcd.print(datumCas.day());                    // Zapiš den v měsíci na LCD
  }
  if (datumCas.month() < 10) {                    // Pokud je měsíc menší jak 10,otevři větev
    lcd.setCursor(8, 0);                          // Nastav LCD na 0 řádek, 8 pozice
    lcd.print("0");                               // Zapiš 0 před mesíc na LCD
    lcd.print(datumCas.month());                  // Zapiš měsíc na LCD
  }                                               // Ukonči větev
  if (datumCas.month() > 9) {                     // Nebo je-li měsíc větší jak 9 otevři větev
    lcd.setCursor(8, 0);                          // Nastav LCD na 0 řádek, 8 pozice
    lcd.print(datumCas.month());                  // Zapiš měsíc na LCD
  }                                               // Ukonči větev

  lcd.setCursor(7, 0);                            // Nastav LCD na 0 řádek, 7 pozice
  lcd.print('.');                                 // Zapiš na LCD tečku za den
  lcd.setCursor(10, 0);                           // Nastav LCD na 0 řádek, 10 pozice
  lcd.print('.');                                 // Zapiš na LCD tečku za měsíc

  lcd.setCursor(11, 0);                           // Nastav LCD na 0 řádek, 11 pozice
  lcd.print(datumCas.year());                     // Zapiš na LCD aktuální rok

   if (WiFi.status() != WL_CONNECTED) {           // Pokud není připojení k WiFi, otevři větev  
    digitalWrite(D10, LOW);                       // Zhasni modrou LED na pinu D10
 //   lcd.setCursor(14, 1);                       // Nastav LCD na 1 řádek, 14 pozice
 //   lcd.print("--");                            // Zapiš na LCD znaky --
  }                                               // Zavři větev
  if (WiFi.status() == WL_CONNECTED) {            // Pokud je připojení k WiFi, otevři větev
    digitalWrite(D10, HIGH);                      // Rozsviť modrou LED na pinu D10  
 //   lcd.setCursor(14, 1);                       // Nastav LCD na 1 řádek,14 pozice
 //   lcd.print("Ok");                            // Zapiš na LCD znaky Ok
 //   lcd.setCursor(0, 1);                        // Nastav LCD na 1 řádek, 0 pozice
 //   lcd.print(pocet);                           // Zapiš na LCD počet znovupřipojení k WiFi
  }                                               // Zavři větev
  
  if (synchronizace=true) {                       // Pokud probíha synchronizace času, otevři větev
    taktu=taktu+1;                                // do pomocného počítadla připočti 1 každý cyklus CPU
  }                                               // Zavři větev

  if (taktu >50)   {                              // Pokud CPU během synchronizace udělalo více jak 50 taktů, otevři větev                      
    synchronizace=false;                          // resetuj pomocný bit synchronizace probíhá,
    taktu=0;                                      // vynuluj pomocné počítadlo počet cyklů CPU během synchronizace
    digitalWrite(D9, LOW);                        // a zhasni žlutou LED na pinu D9
  }                                               // Zavři větev
}                                                 // Zavři větev

void zapisdat() {                                 // Funkce Zápis dat, otevři větev
  if (WiFi.status() == WL_CONNECTED) {            // Je-li připojení k WiFi aktivní, zavolej funkci, otevři větev
  ntp();                                          // NTP
}                                                 // Zavři větev
}                                                 // Zavři větev

void ntp() {                                      // Funkce NTP, otevři větev

int hodina;                                       // Proměnná hodina jako Integer
int minuta;                                       // Proměnná minuta jako Integer
int vterina;                                      // Proměnná vteřina jako Integer


  formattedDate = timeClient.getFormattedDate();  // Deklarace formátovaného datumu a času

  // Extrahuj datum
  int splitT = formattedDate.indexOf("T");        // Index formátovaného datumu a času
  dayStamp = formattedDate.substring(0, splitT);  // Ukazatel na pole datumu
  timeStamp = formattedDate.substring(splitT+1, formattedDate.length()-1); // Ukazatel na pole času

String year = dayStamp.substring(0,4);     // Rok mezi indexy 0 až 4
String month = dayStamp.substring(5,7);    // Měsíc mezi indexy 5 až 7
String day = dayStamp.substring(8);        // Den od indexu 8

int rok=year.toInt();                      // Konverze roku do Integer
int mesic=month.toInt();                   // Konverze měsíce do Integer
int den=day.toInt();                       // Konverze dne do Integer

  
hodina=timeClient.getHours();              // Vyčtení hodin
minuta=timeClient.getMinutes();            // Vyčtení minut
vterina=timeClient.getSeconds();           // Vyčtení vteřin

pocet=pocet+1;                             // Pomocné počítadlo
DS1307.adjust(DateTime((rok), (mesic), (den) , (hodina), (minuta), (vterina)));  // Nastavení aktuálního datumu a času do RTC modulu
synchronizace=true;                        // Zapnutí pomocného bitu synchronizace probíhá
digitalWrite(D9, HIGH);                    // Rozsviť žlutou LED na pinu D9
}                                          // Uzavři větev, konec programu
